#include "esp_vfs_fat.h"
#include "freertos/portmacro.h"
#include "sdmmc_cmd.h"
#include "driver/sdmmc_host.h"

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/unistd.h>
#include <dirent.h>

#define MAX_CHAR_SIZE 128

static const char *TAG = "SPI BUS Aquisition : ";

#define MOUNT_POINT "/sdcard"
#define Flash_Access_Frequency CONFIG_HOST_MAX_FREQUENCY

#define SPI_THRESHOLD CONFIG_SPI_BUS_TIME_THRESHOLD
#define SPI_TASK_FREQUENCY CONFIG_SPI_BUS_TASK_FREQUENCY


sdmmc_card_t *card;
sdmmc_host_t host = SDSPI_HOST_DEFAULT();
const char mount_point[] = MOUNT_POINT;

volatile int64_t start_time = 0 ; 
volatile int64_t elapsed_time = 0 ; 
bool Can_TakeOver_SPI_BUS = false ; 


int List_Files_ON_Flash()
{
  DIR *d ; 
  struct dirent *dir; 
  d = opendir(MOUNT_POINT); 
  if (d) 
  {
    printf("###############################\n");
    printf("### Listing Files on Flash ..\n");
    printf("###############################\n");
    
    int rank = 1 ;
    while ((dir = readdir(d)) != NULL) 
      {
        printf("%d. %s\n",rank, dir->d_name);
        rank += 1; 
      }

    printf("###############################\n\n");
    closedir(d);

    ESP_LOGI(TAG, "File written");
  }
  return 0; 
}

int Init_SPI_BUS() 
{
  esp_err_t ret;

  esp_vfs_fat_sdmmc_mount_config_t mount_config = {
      .format_if_mount_failed = false,
      .max_files = 5,
      .allocation_unit_size = 16 * 1024};

  ESP_LOGI(TAG, "Initializing SD card");

  ESP_LOGI(TAG, "Using SPI peripheral");

  sdmmc_host_t host = SDMMC_HOST_DEFAULT();
  host.max_freq_khz = Flash_Access_Frequency ; // 1000;

  // This initializes the slot without card detect (CD) and write protect (WP) signals.
  // Modify slot_config.gpio_cd and slot_config.gpio_wp if your board has these signals.
  sdmmc_slot_config_t slot_config = SDMMC_SLOT_CONFIG_DEFAULT();

  // Set bus width to use:
  slot_config.width = 1;
  slot_config.flags |= SDMMC_SLOT_FLAG_INTERNAL_PULLUP;

    ESP_LOGI(TAG, "Mounting filesystem");
    ret = esp_vfs_fat_sdmmc_mount(mount_point, &host, &slot_config, &mount_config, &card);

  if (ret != ESP_OK) 
  {
    if (ret == ESP_FAIL) 
    {
      ESP_LOGE(TAG, "Failed to mount filesystem. "
                    "If you want the card to be formatted, set the "
                    "CONFIG_FORMAT_IF_MOUNT_FAILED menuconfig option.");
    } 
    else 
    {
      ESP_LOGE(TAG,
               "Failed to initialize the card (%s). "
               "Make sure SD card lines have pull-up resistors in place.",
               esp_err_to_name(ret));
    }
    return -1 ;
  }
  ESP_LOGI(TAG, "Filesystem mounted");

  printf("###############################\n");
  // Card has been initialized, print its properties
  sdmmc_card_print_info(stdout, card);

  printf("###############################\n");

  return 0 ; 
}

int Free_SPI_BUS()
{
  if (esp_vfs_fat_sdcard_unmount(mount_point, card) == ESP_OK)
    ESP_LOGI(TAG, "\033[1;33mSuccessfully Unmounted sdCard !\033[0m");

  return 0; 
}

void Task_TakeOver_SPI_BUS(void* arrow)
{
  for(;;)
    {
        ESP_LOGI(TAG, "###### Exceeded Threshold Time #######");

        Init_SPI_BUS();
        // List_Files_ON_Flash(); 
        Free_SPI_BUS();
        
        heap_caps_check_integrity_all(true);
				ESP_LOGI(TAG,"\033[1;33m ************************\033[0m");
				ESP_LOGI(TAG,"\033[1;33m Free memory: %0.3f Kb\033[0m ", esp_get_free_heap_size()/1000.0);
				ESP_LOGI(TAG,"\033[1;33m ************************\033[0m");
  
      vTaskDelay( 2000 / portTICK_PERIOD_MS); 
    }
}

void app_main(void) 
{
  // esp_log_level_set("gpio", ESP_LOG_ERROR);
  // esp_log_level_set("sdspi_transaction", ESP_LOG_ERROR);

  xTaskCreate(Task_TakeOver_SPI_BUS, "xTask_SPI", 2048, NULL, 5, NULL); 
  
}
